const Mesa = require('vendor/Mesa.js');

/**
 * A Mesa Script exports a class with a script() method.
 */
module.exports = new class {

  /**
   * Mesa Script
   *
   * @param {object} payload The payload data
   * @param {object} context Additional context about this task
   */
  script = (payload, context) => {
    const vars = context.steps;

    let parentInventory = vars.webhook.shopify.available;
    let previousParentInventory = this.previousParentInventory(vars);
    let parentInventoryDelta = parentInventory - previousParentInventory;

    let childQuantityPerBundle = vars.loop.data['Child Quantity'];
    let childCurrentInventory = vars.shopify_3.inventory_quantity;
    let childNewInventory = childCurrentInventory + (parentInventoryDelta * childQuantityPerBundle);

    Mesa.trigger.setTaskExternalData({
      "label": `Parent Previous: ${previousParentInventory}, 
                Parent Current: ${parentInventory},
                Parent Delta: ${parentInventoryDelta},
                Child Current: ${childCurrentInventory},
                Child New: ${childNewInventory}`
    });

    Mesa.output.next({"available": childNewInventory});
  }

  previousParentInventory = (vars) => {
    const skuToQuantityObject = vars.custom_2;

    let lowestQuantity = Number.MAX_SAFE_INTEGER;
    let childQuantityPerBundle = null;
    let childQuantityDivided = null;

    for (const [sku, childQuantity] of Object.entries(skuToQuantityObject)) {
      childQuantityPerBundle = this.childQuantityPerBundle(vars, sku);
      childQuantityDivided = Math.floor(childQuantity / childQuantityPerBundle);
      Mesa.log.info(`=============
        Child quantity per bundle: ${sku} - quantity: ${childQuantity}, quantity per bundle: ${childQuantityPerBundle}, divided: ${childQuantityDivided}`
      );
      if (childQuantityDivided < lowestQuantity) {
        lowestQuantity = childQuantityDivided;
      }
    }

    return lowestQuantity;
  }

  childQuantityPerBundle(vars, childSku) {
    const childSkuToQuantityObject = vars.custom.reduce((accumulator, item) => {
      const { "Child SKU": childSku, "Child Quantity": childQuantity } = item.data;
      accumulator[childSku] = childQuantity;
      return accumulator;
    }, {});

    return childSkuToQuantityObject[childSku];
  }
}
