const Mesa = require('vendor/Mesa.js');
const ShopifyGraphql = require('vendor/ShopifyGraphql.js');
const Shopify= require('vendor/Shopify.js');

/**
 * A Mesa Script exports a class with a script() method.
 */
module.exports = new class {
  script = (payload, context) => {
    let vars = context.steps;
    
    let query = `
      query productVariants($query:String!) {
        productVariants(query:$query, first:10) {
          nodes {
            id
            sku
            inventoryQuantity
          }
        }
      }
    `;
    
    let skuQuery = vars.custom.map(
      item => `sku:${item.data['Child SKU']}`
    ).join(' OR ');

    const r = ShopifyGraphql.send(query, {
      "query": skuQuery,
    }, {}, 'admin/api/2023-10/graphql.json');

    const skuToQuantityObject = r.data.productVariants.nodes.reduce((accumulator, item) => {
      accumulator[item.sku] = item.inventoryQuantity;
      return accumulator;
    }, {});

    Mesa.trigger.setTaskExternalData({
      "label": JSON.stringify(skuToQuantityObject)
    })

    Mesa.output.next(skuToQuantityObject);
  }
}
